package com.zanxiang.game.back.serve.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.back.serve.pojo.dto.*;
import com.zanxiang.game.back.serve.pojo.vo.*;
import com.zanxiang.game.back.serve.service.*;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/policy")
@Api("回传策略")
public class BackPolicyController {
    @Autowired
    private IGameBackPolicyService gameBackPolicyService;

    @ApiOperation(value = "创建回传策略")
    @PreAuthorize(permissionKey = "gameBack:policy:allOfUser")
    @GetMapping("/allOfUser")
    public ResultVO<List<GameBackPolicyVO>> allOfUser() {
        return ResultVO.ok(gameBackPolicyService.allOfUser());
    }

    @PreAuthorize(permissionKey = "gameBack:policy:list")
    @PostMapping("/list")
    @ApiOperation(value = "回传策略列表")
    public ResultVO<IPage<GameBackPolicyVO>> policyList(@RequestBody GameBackPolicyListDTO dto) {
        return ResultVO.ok(gameBackPolicyService.policyList(dto));
    }

    @PreAuthorize(permissionKey = "gameBack:policy:create")
    @PostMapping("/create")
    @ApiOperation(value = "创建回传策略")
    public ResultVO<Boolean> createPolicy(@RequestBody GameBackPolicyDTO dto) {
        return ResultVO.ok(gameBackPolicyService.createPolicy(dto));
    }

    @PreAuthorize(permissionKey = "gameBack:policy:update")
    @PostMapping("/update")
    @ApiOperation(value = "修改回传策略")
    public ResultVO<Boolean> updatePolicy(@RequestBody GameBackPolicyDTO dto) {
        return ResultVO.ok(gameBackPolicyService.updatePolicy(dto));
    }

    @PreAuthorize(permissionKey = "gameBack:policy:delete")
    @PostMapping("/delete/{ids}")
    @ApiOperation(value = "删除回传策略")
    public ResultVO<Boolean> deletePolicy(@PathVariable("ids") List<Long> ids) {
        return ResultVO.ok(gameBackPolicyService.deletePolicy(ids));
    }
}
