package com.zanxiang.game.back.serve.rpc.impl;

import com.zanxiang.game.back.base.oceanengine.pojo.dto.OrderReportRpcDTO;
import com.zanxiang.game.back.base.oceanengine.pojo.dto.UserActiveReportRpcDTO;
import com.zanxiang.game.back.base.oceanengine.rpc.IWechatMiniGameDataReportRpc;
import com.zanxiang.game.back.serve.pojo.entity.GameOceanengineOrderLog;
import com.zanxiang.game.back.serve.pojo.entity.GameOceanengineUserLog;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameOceanengineOrderLogService;
import com.zanxiang.game.back.serve.service.IGameOceanengineUserLogService;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

@Slf4j
@DubboService
public class WechatMiniGameDataReportRpcImpl implements IWechatMiniGameDataReportRpc {
    @Autowired
    private IGameOceanengineUserLogService gameOceanengineUserLogService;
    @Autowired
    private IGameOceanengineOrderLogService gameOceanengineOrderLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> userActiveReport(UserActiveReportRpcDTO dto) {
        GameOceanengineUserLog userLog = GameOceanengineUserLog.builder()
                .appId(dto.getWechatAppId())
                .openId(dto.getWechatOpenId())
                .unionId(dto.getWechatUnionId())
                .gameId(dto.getGameId())
                .accountId(dto.getAccountReport().getAccountId())
                .accountReportToken(dto.getAccountReport().getReportToken())
                .accountReportUrl(dto.getAccountReport().getReportUrl())
                .agentKey(dto.getAgentKey())
                .clueToken(dto.getClueToken())
                .projectId(dto.getProjectId())
                .promotionId(dto.getPromotionId())
                .reqId(dto.getReqId())
                .activeTime(dto.getActiveTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .build();
        gameOceanengineUserLogService.save(userLog);
        return ResultVO.ok(gameOceanengineUserLogService.callback(userLog, false));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> orderReport(OrderReportRpcDTO dto) {
        GameOceanengineOrderLog orderLog = GameOceanengineOrderLog.builder()
                .appId(dto.getWechatAppId())
                .openId(dto.getWechatOpenId())
                .unionId(dto.getWechatUnionId())
                .gameId(dto.getGameId())
                .backPolicyId(dto.getBackPolicyId())
                .accountId(dto.getAccountReport().getAccountId())
                .accountReportToken(dto.getAccountReport().getReportToken())
                .accountReportUrl(dto.getAccountReport().getReportUrl())
                .agentKey(dto.getAgentKey())
                .orderNo(dto.getOrderId())
                .amount(dto.getRechargeMoney())
                .orderStatus(dto.getOrderStatus())
                .createTime(dto.getCreateTime())
                .payTime(dto.getPayTime())
                .backStatus(BackStatusEnum.NO.getBackStatus())
                .build();
        gameOceanengineOrderLogService.save(orderLog);
        return ResultVO.ok(gameOceanengineOrderLogService.callback(orderLog, false));
    }
}
