package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.pojo.dto.DataReportOfAppIdRpcDTO;
import com.zanxiang.advertising.tencent.base.pojo.dto.UserActionRpcDTO;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.back.serve.pojo.dto.GameBackPolicyListDTO;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentOrderDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentOrder;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentOrderMapper;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentOrderVO;
import com.zanxiang.game.back.serve.service.IGameBackPolicyService;
import com.zanxiang.game.back.serve.service.IGameTencentBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import static com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum.PURCHASE;

/**
 * <p>
 * 游戏腾讯订单表 服务实现类
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Slf4j
@Service
public class GameTencentOrderServiceImpl extends ServiceImpl<GameTencentOrderMapper, GameTencentOrder> implements IGameTencentOrderService {
    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;
    @Autowired
    private IGameTencentBackLogService gameTencentBackLogService;
    /**
     * 支付状态，0 : 预下单, 1 : 待支付，2 : 支付成功，-1 : 已取消
     */
    private static final Integer COMPLETE_ORDER = 0;
    private static final Integer PURCHASE = 2;


    @Override
    public IPage<GameTencentOrderVO> tencentOrderLogList(GameTencentOrderDTO dto) {
        return page(dto.toPage(), new LambdaQueryWrapper<GameTencentOrder>()
                .eq(StringUtils.isNotBlank(dto.getChannel()), GameTencentOrder::getChannel, dto.getChannel())
                .eq(null != dto.getGameId(), GameTencentOrder::getGameId, dto.getGameId())
                .eq(null != dto.getAdAccountId(), GameTencentOrder::getAdAccountId, dto.getAdAccountId())
                .eq(StringUtils.isNotBlank(dto.getWechatAppId()), GameTencentOrder::getWechatAppId, dto.getWechatAppId())
                .eq(StringUtils.isNotBlank(dto.getWechatOpenid()), GameTencentOrder::getWechatOpenid, dto.getWechatOpenid())
                .eq(StringUtils.isNotBlank(dto.getOrderId()), GameTencentOrder::getOrderId, dto.getOrderId())
                .eq(null != dto.getIsBack(), GameTencentOrder::getIsBack, dto.getIsBack())
                .eq(null != dto.getOrderStatus(), GameTencentOrder::getOrderStatus, dto.getOrderStatus())
                .orderByDesc(GameTencentOrder::getCreateTime)
        ).convert(item -> {
            GameTencentOrderVO copy = BeanUtil.copy(item, GameTencentOrderVO.class);
            copy.setUpdateName(null == item.getUpdateBy() ? null : sysUserRpc.getById(item.getUpdateBy()).getData().getNickName());
            return copy;
        });
    }

    @Override
    public boolean tencentOrderReport(List<Long> ids) {
        baseMapper.selectBatchIds(ids).stream()
                .filter(item -> !item.getIsBack().equals(BackStatusEnum.SUCCESS.getBackStatus()))
                .forEach(gameTencentOrder -> {
                    String actionType = gameTencentOrder.getOrderStatus().equals(PURCHASE) ?
                            ActionTypeEnum.PURCHASE.getActionType() : ActionTypeEnum.COMPLETE_ORDER.getActionType();
                    DataReportOfAppIdRpcDTO dataReportOfAppIdRpcDTO = DataReportOfAppIdRpcDTO.builder()
                            .appId(gameTencentOrder.getWechatAppId())
                            .action(UserActionRpcDTO.builder()
                                    .actionTime(gameTencentOrder.getOrderStatus().equals(PURCHASE) ? gameTencentOrder.getPayTime() : gameTencentOrder.getRechargeTime())
                                    .actionType(actionType)
                                    .userId(UserActionRpcDTO.UserIdRpcDTO.builder()
                                            .wechatAppId(gameTencentOrder.getWechatAppId())
                                            .wechatOpenid(gameTencentOrder.getWechatOpenid())
                                            .build())
                                    .build())
                            .build();

                    GameTencentBackLog gameTencentBackLog = GameTencentBackLog.builder()
                            .gameId(gameTencentOrder.getGameId())
                            .adAccountId(gameTencentOrder.getAdAccountId())
                            .actionTime(gameTencentOrder.getOrderStatus().equals(PURCHASE) ? gameTencentOrder.getPayTime() : gameTencentOrder.getRechargeTime())
                            .createTime(LocalDateTime.now())
                            .actionType(actionType)
                            .orderId(gameTencentOrder.getOrderId())
                            .wechatAppId(gameTencentOrder.getWechatAppId())
                            .wechatOpenid(gameTencentOrder.getWechatOpenid())
                            .build();
                    gameTencentOrder.setUpdateTime(LocalDateTime.now());
                    gameTencentOrder.setUpdateBy(SecurityUtil.getUserId());
                    try {
                        userActionSetRpc.reportByAppId(dataReportOfAppIdRpcDTO);
                        gameTencentOrder.setIsBack(BackStatusEnum.SUCCESS.getBackStatus());
                        updateById(gameTencentOrder);

                        gameTencentBackLogService.save(gameTencentBackLog);
                    } catch (Exception e) {
                        log.error("回传腾讯订单失败，失败原因：{}", e.getMessage());
                        gameTencentOrder.setBackLog(e.getMessage());
                        gameTencentOrder.setIsBack(BackStatusEnum.FAILED.getBackStatus());
                        updateById(gameTencentOrder);

                        gameTencentBackLog.setBackLog(e.getMessage());
                        gameTencentBackLogService.saveOrUpdate(gameTencentBackLog);
                    }
                });
        return true;
    }
}
