package com.zanxiang.game.back.serve.utils;

import com.github.sd4324530.jtuple.Tuple2;
import com.github.sd4324530.jtuple.Tuples;
import com.zanxiang.game.back.serve.pojo.entity.GameBackPolicy;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Random;
import java.util.function.Function;

public class BackPolicyUtil {
    private static final long[] RECHARGE_LEVEL = {12 * 100L, 30 * 100L, 50 * 100L, 98 * 100L, 198 * 100L, 328 * 100L, 649 * 100L, 998 * 100L, 2998 * 100L};

    static {
        Arrays.sort(RECHARGE_LEVEL);
    }

    public static boolean isBackOrder(GameBackPolicy gameBackPolicy, Long rechargeMoney) {
        if (null == gameBackPolicy) {
            return true;
        } else {
            if (rechargeMoney <= gameBackPolicy.getFirstRechargeMoney() && shouldDoTask(gameBackPolicy.getFirstRechargeBackPercentage())) {
                return true;
            } else if (rechargeMoney >= gameBackPolicy.getFirstRechargeMoney() && shouldDoTask(gameBackPolicy.getBigRechargeBackPercentage())) {
                return true;
            } else {
                return shouldDoTask(gameBackPolicy.getOtherRechargeBackPercentage());
            }
        }
    }

    private static boolean shouldDoTask(BigDecimal successProbability) {
        Random random = new Random();
        // 生成一个0到1之间的随机数
        double randomValue = random.nextDouble();
        // 如果随机数小于成功概率，则返回true，否则返回false
        return successProbability.compareTo(BigDecimal.valueOf(randomValue)) >= 0;
    }

    /**
     * 订单回传
     *
     * @param gameBackPolicy   回传策略
     * @param rechargeMoney    充值金额
     * @param lastBackFunction 在指定最近订单数量内是否有订单回传过
     * @return <是否回传, 回传金额>
     */
    public static boolean backOrder(GameBackPolicy gameBackPolicy, long rechargeMoney, Function<Integer, Integer> lastBackFunction) {
        if (null == gameBackPolicy) {
            return true;
        }
        BigDecimal percentage;
        if (rechargeMoney <= gameBackPolicy.getFirstRechargeMoney()) {
            // 小于首充金额
            percentage = gameBackPolicy.getFirstRechargeBackPercentage();
        } else if (rechargeMoney >= gameBackPolicy.getBigRechargeMoney()) {
            percentage = gameBackPolicy.getBigRechargeBackPercentage();
        } else {
            percentage = gameBackPolicy.getOtherRechargeBackPercentage();
        }
        if (percentage == null || percentage.compareTo(BigDecimal.ONE) >= 0) {
            return true;
        }
        int backRate = percentage.multiply(BigDecimal.TEN).intValue();
        Tuple2<Integer, Integer> temp = approximate(backRate, 10);
        backRate = temp.first;
        int backUnit = temp.second;
        int backCount = lastBackFunction.apply(backUnit);
        if (backCount >= backRate) {
            // 之前有订单上报过，这笔订单不上报了
            return false;
        } else {
            return true;
        }
    }

    private static Tuple2<Integer, Integer> approximate(int molecular, int denominator) {
        if (molecular % denominator == 0) {
            int temp = molecular / denominator;
            return Tuples.tuple(temp, temp);
        }
        int model = 2;
        do {
            if (molecular % model == 0 && denominator % model == 0) {
                molecular = molecular / model;
                denominator = denominator / model;
                continue;
            }
            model++;
        } while (model <= denominator / 2);
        return Tuples.tuple(molecular, denominator);
    }

    /**
     * 降档
     */
    public static long lowRechargeLevel(long rechargeMoney, Integer levelDown) {
        if (levelDown == null || levelDown == 0) {
            return rechargeMoney;
        }
        int level = RECHARGE_LEVEL.length - 1;
        for (int i = 0; i < RECHARGE_LEVEL.length; i++) {
            if (rechargeMoney <= RECHARGE_LEVEL[i]) {
                level = i;
                break;
            }
        }
        level = level - levelDown;
        level = level < 0 ? 0 : Math.min(level, RECHARGE_LEVEL.length - 1);
        return RECHARGE_LEVEL[level];
    }
}
