package com.zanxiang.game.data.serve.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.data.serve.dao.mapper.*;
import com.zanxiang.game.data.serve.pojo.dto.GameOrderDetailDTO;
import com.zanxiang.game.data.serve.pojo.entity.*;
import com.zanxiang.game.data.serve.pojo.vo.GameOrderDetailVO;
import com.zanxiang.game.data.serve.pojo.vo.OrderDetailVO;
import com.zanxiang.game.data.serve.pojo.vo.TotalDataVO;
import com.zanxiang.game.data.serve.service.IGameOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.data.serve.service.IPromoteAgentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Objects;

/**
 * <p>
 * t_game_order 服务实现类
 * </p>
 *
 * @author auto
 * @since 2022-12-08
 */
@Service
public class GameOrderServiceImpl extends ServiceImpl<GameOrderMapper, GameOrder> implements IGameOrderService {
    @Autowired
    private GameUserMapper gameUserMapper;
    @Autowired
    private PromoteAgentMapper promoteAgentMapper;
    @Autowired
    private GameMapper gameMapper;
    @Autowired
    private AdAccountMapper adAccountMapper;
    @Autowired
    private GamePlayerRoleMapper gamePlayerRoleMapper;
    @Autowired
    private GameServerMapper gameServerMapper;
    @Autowired
    private SysUserMapper sysUserMapper;

    @Override
    public GameOrderDetailVO getOrderDetail(GameOrderDetailDTO dto) {
        Long sysUserId = SecurityUtil.isAdmin() ? null : SecurityUtil.getUserId();
        sysUserId = null == sysUserId ? null : dto.getSysUserId();
        dto.setSysUserId(sysUserId);
        IPage<OrderDetailVO> orderDetail = getBaseMapper().getOrderDetail(dto.toPage(), dto);
        TotalDataVO amountSum = getBaseMapper().getAmountSum(dto);

        orderDetail.getRecords().forEach(item->{
            Game regGame = gameMapper.selectById(item.getRegGameId());
            if (Objects.nonNull(regGame)){
                item.setRegGameCategoryName(regGame.getCategory());
                item.setUserRegGameName(regGame.getGameName());
            }
            Game rechargeGame = gameMapper.selectById(item.getRechargeGameId());
            if (Objects.nonNull(rechargeGame)){
                item.setRegGameCategoryName(rechargeGame.getCategory());
                item.setUserRegGameName(rechargeGame.getGameName());
            }
            SysUser sysUser = sysUserMapper.selectById(item.getPitcherId());
            if (Objects.nonNull(sysUser)){
                item.setPitcherName(sysUser.getNickname());
            }
            PromoteAgent rechargeAgent = promoteAgentMapper.selectById(item.getRechargeAgentId());
            if (Objects.nonNull(rechargeAgent)) {
                item.setRechargeAgentName(rechargeAgent.getAgentName());
                item.setMediaName(rechargeAgent.getPromoteMedia());
            }
            GameUser gameUser = gameUserMapper.selectById(item.getUserId());
            if (Objects.nonNull(gameUser)){
                PromoteAgent userAgent = promoteAgentMapper.selectById(gameUser.getAgentId());
                if (Objects.nonNull(userAgent)) {
                    item.setUserRegAgentName(userAgent.getAgentName());
                    item.setUserRegAgentId(userAgent.getId());
                }
            }
            GameServer gameServer = gameServerMapper.selectById(item.getServerId());
            if (Objects.nonNull(gameServer)){
                item.setServerName(gameServer.getServerName());
            }
            GamePlayerRole gamePlayerRole = gamePlayerRoleMapper.selectById(item.getRoleId());
            if (Objects.nonNull(gamePlayerRole)){
                item.setRoleLevel(gamePlayerRole.getRoleLevel());
                item.setRoleName(gamePlayerRole.getRoleName());
                item.setRoleVipLevel(gamePlayerRole.getRoleVip());
            }
        });
        return GameOrderDetailVO.builder()
                .totalData(TotalDataVO.builder()
                        .totalAmount(Objects.nonNull(amountSum)?amountSum.getTotalAmount() : BigDecimal.valueOf(0))
                        .totalRealAmount(Objects.nonNull(amountSum)?amountSum.getTotalRealAmount() : BigDecimal.valueOf(0))
                        .build())
                .records(orderDetail.getRecords())
                .current(orderDetail.getCurrent())
                .pages(orderDetail.getPages())
                .size(orderDetail.getSize())
                .total(orderDetail.getTotal())
                .build();
    }
}
