package com.zanxiang.game.data.serve.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.data.serve.pojo.dto.GameRankingDTO;
import com.zanxiang.game.data.serve.pojo.entity.Game;
import com.zanxiang.game.data.serve.dao.mapper.GameMapper;
import com.zanxiang.game.data.serve.pojo.vo.GameConditionVO;
import com.zanxiang.game.data.serve.pojo.vo.GameRankingRechargeGameVO;
import com.zanxiang.game.data.serve.service.IGameService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.data.serve.service.IPromoteAgentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * t_game 服务实现类
 * </p>
 *
 * @author auto
 * @since 2022-12-08
 */
@Service
public class GameServiceImpl extends ServiceImpl<GameMapper, Game> implements IGameService {
    @Autowired
    private IPromoteAgentService promoteAgentService;

    @Override
    public IPage<GameRankingRechargeGameVO> getGameRechargeRank(GameRankingDTO dto) {
        Long sysUserId = SecurityUtil.isAdmin() ? null : SecurityUtil.getUserId();
        sysUserId = null == sysUserId ? null : dto.getSysUserId();
        dto.setSysUserId(sysUserId);
        IPage<GameRankingRechargeGameVO> gameRankingRechargeVO = getBaseMapper().getGameRankingRechargeVO(dto.toPage(), dto);
        gameRankingRechargeVO.getRecords().forEach(item -> {
                    BigDecimal paybackRate = null == item.getTotalCost() ?
                            BigDecimal.valueOf(0) : item.getTotalPurchaseRechargeMoney().divide(item.getTotalCost(), 4, RoundingMode.DOWN);
                    BigDecimal avgMoney = null == item.getTotalRechargeCount() ?
                            BigDecimal.valueOf(0) : item.getTotalRechargeMoney().divide(BigDecimal.valueOf(item.getTotalRechargeCount()), 4, RoundingMode.DOWN);
                    GameRankingRechargeGameVO todayMoney = getBaseMapper().getTodayMoney(item.getGameId(), LocalDate.now(), dto.getSysUserId());
                    if (Objects.nonNull(todayMoney)) {
                        item.setTodayRechargeMoney(todayMoney.getTodayRechargeMoney());
                        item.setTodayNatureRechargeMoney(todayMoney.getTodayNatureRechargeMoney());
                        item.setTodayPurchaseRechargeMoney(todayMoney.getTodayPurchaseRechargeMoney());
                    }
                    item.setPaybackRate(paybackRate);
                    item.setAvgMoney(avgMoney);
                }
        );

        return gameRankingRechargeVO;
    }

    @Override
    public List<GameConditionVO> getList() {
        return getBaseMapper()
                .selectList(null)
                .stream()
                .map(item -> GameConditionVO
                        .builder()
                        .id(String.valueOf(item.getId()))
                        .name(item.getGameName())
                        .build()).collect(Collectors.toList());
    }
}
