package com.zanxiang.game.data.serve.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.data.serve.dao.mapper.GameUserStatDayMapper;
import com.zanxiang.game.data.serve.pojo.dto.GameRankingDTO;
import com.zanxiang.game.data.serve.pojo.entity.GameUser;
import com.zanxiang.game.data.serve.dao.mapper.GameUserMapper;
import com.zanxiang.game.data.serve.pojo.vo.GameConditionVO;
import com.zanxiang.game.data.serve.pojo.vo.GameRankingRechargeUserVO;
import com.zanxiang.game.data.serve.service.IGameUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * t_game_user 服务实现类
 * </p>
 *
 * @author auto
 * @since 2022-12-08
 */
@Service
public class GameUserServiceImpl extends ServiceImpl<GameUserMapper, GameUser> implements IGameUserService {
    @Autowired
    private GameUserStatDayMapper gameUserStatDayMapper;

    @Override
    public IPage<GameRankingRechargeUserVO> getUserRechargeRank(GameRankingDTO dto) {
        Long sysUserId = SecurityUtil.isAdmin() ? null : SecurityUtil.getUserId();
        sysUserId = null == sysUserId ? null : dto.getSysUserId();
        dto.setSysUserId(sysUserId);
        IPage<GameRankingRechargeUserVO> gameRankingRechargeVO = gameUserStatDayMapper.getGameUserRankingRechargeVO(dto.toPage(), dto);
        gameRankingRechargeVO.getRecords().forEach(item -> {
                    BigDecimal avgMoney = null == item.getTotalRechargeCount() ?
                            BigDecimal.valueOf(0) : item.getTotalRechargeMoney().divide(BigDecimal.valueOf(item.getTotalRechargeCount()), 4, RoundingMode.DOWN);
            GameRankingRechargeUserVO todayMoney = getBaseMapper().getTodayGameUserRecharge(item.getUserId(),item.getRegAgent(), dto.getSysUserId());
            item.setRegAgent(null);
                    if (Objects.nonNull(todayMoney)) {
                        item.setTodayRechargeMoney(todayMoney.getTodayRechargeMoney());
                        item.setUserAccount(todayMoney.getUserAccount());
                        item.setRegAgent(todayMoney.getRegAgent());
                        item.setRegTime(todayMoney.getRegTime());
                        item.setRecentRechargeGame(todayMoney.getRecentRechargeGame());
                        item.setUserDevice(todayMoney.getUserDevice());
                        item.setFirstRechargeMoney(todayMoney.getFirstRechargeMoney());
                        item.setRecentRechargeMoney(todayMoney.getRecentRechargeMoney());
                        item.setTag(todayMoney.getTag());
                        item.setRecentRechargeTime(todayMoney.getRecentRechargeTime());
                    }
                    item.setAvgMoney(avgMoney);
                }
        );

        return gameRankingRechargeVO;
    }

    @Override
    public List<GameConditionVO> getList() {
        return getBaseMapper()
                .selectList(null)
                .stream()
                .map(item -> GameConditionVO
                        .builder()
                        .id(String.valueOf(item.getId()))
                        .name(item.getUsername())
                        .build()).collect(Collectors.toList());
    }
}
