package com.zanxiang.game.data.serve.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.data.serve.pojo.dto.GamePromoteDTO;
import com.zanxiang.game.data.serve.pojo.entity.PromoteAgentStatDay;
import com.zanxiang.game.data.serve.dao.mapper.PromoteAgentStatDayMapper;
import com.zanxiang.game.data.serve.pojo.vo.GamePromoteVO;
import com.zanxiang.game.data.serve.pojo.vo.RechargeTrendVO;
import com.zanxiang.game.data.serve.service.IPromoteAgentStatDayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * <p>
 * 游戏推广渠道日报表 服务实现类
 * </p>
 *
 * @author auto
 * @since 2023-01-04
 */
@Service
public class PromoteAgentStatDayServiceImpl extends ServiceImpl<PromoteAgentStatDayMapper, PromoteAgentStatDay> implements IPromoteAgentStatDayService {

    @Override
    public IPage<GamePromoteVO> getDayData(GamePromoteDTO dto) {
        Long sysUserId = SecurityUtil.isAdmin() ? null : SecurityUtil.getUserId();
        sysUserId = null == sysUserId ? null : dto.getSysUserId();
        dto.setSysUserId(sysUserId);
        IPage<GamePromoteVO> page = getBaseMapper().getDayDate(dto.toPage(), dto);
        page.getRecords().forEach(item -> {
            //回收趋势
            item.setOneDayRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay1())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay1())
                    .addPayBack(BigDecimal.valueOf(0))
                    .payBack(BigDecimal.valueOf(0))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay1().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setThreeDayRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay3())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay3())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay3().subtract(item.getNewUserRechargeMoneyDcDay2())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay3().divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDay3().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setSevenDayRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay7())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay7())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay7().subtract(item.getNewUserRechargeMoneyDcDay6())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay7().divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDay7().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setFifteenDayRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay15())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay15())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay15().subtract(item.getNewUserRechargeMoneyDcDay14())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay15().divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDay15().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setOneMonthRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay30())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay30())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay30().subtract(item.getNewUserRechargeMoneyDcDay29())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay30().divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDay30().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setTwoMonthRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay60())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay60())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay60().subtract(item.getNewUserRechargeMoneyDcDay59())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay60().divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDay60().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setThreeMonthRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUserDcDay90())
                    .rechargeMoney(item.getNewUserRechargeMoneyDcDay90())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay90().subtract(item.getNewUserRechargeMoneyDcDay89())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDcDay90().divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoneyDay90().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());

            item.setThreeMonthRechargeTrend(RechargeTrendVO.builder()
                    .rechargeCount(item.getNewUserRechargeUser())
                    .rechargeMoney(item.getNewUserRechargeMoney())
                    .addPayBack(item.getTotalCost().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : (item.getNewUserRechargeMoneyDcDay90().subtract(item.getNewUserRechargeMoneyDcDay89())).divide(item.getTotalCost(), 4, RoundingMode.DOWN))
                    .payBack(BigDecimal.valueOf(0))
                    .newUserRechargeRate(item.getNewUserRechargeMoneyDcDay1().compareTo(BigDecimal.ZERO) == 0 ?
                            BigDecimal.valueOf(0) : item.getNewUserRechargeMoney().divide(item.getNewUserRechargeMoneyDcDay1(), 4, RoundingMode.DOWN))
                    .build());
        });
        return page;
    }
}
