package com.zanxiang.game.module.manage.pojo.params;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.zanxiang.game.module.mybatis.entity.User;
import com.zanxiang.module.web.pojo.BaseListDTO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.format.annotation.DateTimeFormat;

import java.time.LocalDate;

/**
 * @author : lingfeng
 * @time : 2022-07-06
 * @description : 玩家列表查询参数
 */
@ApiModel
@Data
@EqualsAndHashCode(callSuper = true)
public class UserListParam extends BaseListDTO<User> {

    /**
     * 玩家id
     */
    @ApiModelProperty(notes = "玩家id")
    private Long userId;

    /**
     * 玩家账号
     */
    @ApiModelProperty(notes = "玩家账号")
    private String userName;

    /**
     * 玩家昵称
     */
    @ApiModelProperty(notes = "玩家昵称")
    private String nickname;

    /**
     * 绑定手机号码
     */
    @ApiModelProperty(notes = "绑定手机号码")
    private String mobile;

    /**
     * 真实姓名
     */
    @ApiModelProperty(notes = "真实姓名")
    private String realName;

    /**
     * 注册IP
     */
    @ApiModelProperty(notes = "注册IP")
    private String regIp;

    /**
     * 是否实名认证
     */
    @ApiModelProperty(notes = "是否实名认证")
    private Boolean isAuth;

    /**
     * 玩家手机绑定状态
     */
    @ApiModelProperty(notes = "玩家手机绑定状态")
    private Boolean isBindMobile;

    /**
     * cp名称对应的id
     */
    @ApiModelProperty(notes = "cp名称对应的id")
    private Long cpId;

    /**
     * 游戏名称对应的id
     */
    @ApiModelProperty(notes = "游戏名称对应的id")
    private Long gameId;

    /**
     * 游戏应用类型对应的id
     */
    @ApiModelProperty(notes = "游戏应用类型对应的id")
    private Long gameCategoryId;

    /**
     * 最近充值开始时间(开始玩开始时间请使用beginDate参数)
     */
    @ApiModelProperty(notes = "最近充值开始时间(注册开始时间请使用beginDate参数)")
    private LocalDate rechargeBeginDate;

    /**
     * 最近充值结束时间(开始玩结束时间请使用endDate参数)
     */
    @ApiModelProperty(notes = "最近充值结束时间(注册结束时间请使用endDate参数)")
    private LocalDate rechargeEndDate;

    /**
     * 是否充值
     */
    @ApiModelProperty(notes = "是否充值, true : 是, false : 否, 全部 : null或者不传")
    private Boolean isRecharge;

    /**
     * 角色vip
     */
    @ApiModelProperty(notes = "角色vip, 全部 : null或者不传")
    private Integer vipLevel;

    /**
     * 玩家状态, 0 : 正常, 1 : 封禁
     */
    @ApiModelProperty(notes = "玩家状态, 0 : 正常, 1 : 封禁")
    private Integer status;

    /**
     * 注册渠道对应的id
     */
    @ApiModelProperty(notes = "注册渠道对应的id")
    private Long channelId;

    /**
     * 归因推广账号
     */
    @ApiModelProperty(notes = "归因推广账号")
    private String accountId;

    /**
     * 归因投放人员
     */
    @ApiModelProperty(notes = "归因投放人员")
    private String pitcherId;

    /**
     * 客服id
     */
    @ApiModelProperty(notes = "客服id")
    private Long customerId;

    /**
     * 归因广告id
     */
    @ApiModelProperty(notes = "归因广告id")
    private String adId;

    /**
     * 是否切量
     */
    @ApiModelProperty(notes = "是否切量")
    private Boolean isCutCount;

    /**
     * 开始时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ApiModelProperty(value = "开始时间")
    private LocalDate beginDate;

    /**
     * 结束时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ApiModelProperty(value = "结束时间")
    private LocalDate endDate;
}
