package com.zanxiang.game.module.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.game.module.manage.pojo.dto.GameDTO;
import com.zanxiang.game.module.manage.pojo.params.GameAddParam;
import com.zanxiang.game.module.manage.pojo.params.GameListParam;
import com.zanxiang.game.module.manage.pojo.vo.GameChoiceVO;
import com.zanxiang.game.module.manage.pojo.vo.GameInfoVO;
import com.zanxiang.game.module.manage.pojo.vo.GameListVO;
import com.zanxiang.game.module.mybatis.entity.Game;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-24
 * @description : 游戏逻辑
 */
public interface IGameService extends IService<Game> {

    /**
     * 游戏添加或更新
     *
     * @param param : 游戏新增参数
     * @return {@link Boolean}
     */
    Boolean gameAddOrUpdate(GameAddParam param);

    /**
     * 查询游戏列表
     *
     * @param param : 游戏列表查询参数
     * @return {@link IPage}<{@link GameListVO}>
     */
    IPage<GameListVO> gameList(GameListParam param);

    /**
     * 游戏选择列表查询
     *
     * @return {@link List}<{@link GameChoiceVO}>
     */
    List<GameChoiceVO> choiceList();

    /**
     * 信息由ids
     *
     * @param gameIds 游戏id
     * @return {@link List}<{@link GameInfoVO}>
     */
    List<GameInfoVO> infoByIds(Collection<Long>  gameIds);

    /**
     * 根据id查询游戏信息
     *
     * @param id : 游戏id
     * @return {@link GameDTO}
     */
    GameDTO getById(Long id);

    /**
     * 游戏相关条件查询游戏信息
     *
     * @param cpId           : cp唯一键
     * @param gameId         : 游戏id
     * @param gameCategoryId : 分类id
     * @return {@link Map}<{@link Long}, {@link GameDTO}>
     */
    Map<Long, GameDTO> gameCondition(Long cpId, Long gameId, Long gameCategoryId);

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return {@link Boolean}
     */
    Boolean deleteById(Long id);

    /**
     * 根据主键更新状态
     *
     * @param id     : 主键id
     * @param status : 状态
     * @return {@link Boolean}
     */
    Boolean statusUpdate(Long id, Integer status);

    /**
     * 游戏列表
     *
     * @param gameIds 游戏id列表
     * @return {@link Map}<{@link Long}, {@link GameDTO}>
     */
    Map<Long, GameDTO> gameMap(Collection<Long> gameIds);

    /**
     * 游戏列表
     *
     * @return {@link Map}<{@link Long}, {@link GameDTO}>
     */
    Map<Long, GameDTO> gameMap();
}
