package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.manage.pojo.dto.AgentDTO;
import com.zanxiang.game.module.manage.pojo.dto.UserDTO;
import com.zanxiang.game.module.manage.pojo.params.UserNameAuthListParam;
import com.zanxiang.game.module.manage.pojo.vo.UserCardVO;
import com.zanxiang.game.module.manage.service.IAgentService;
import com.zanxiang.game.module.manage.service.IUserCardService;
import com.zanxiang.game.module.manage.service.IUserService;
import com.zanxiang.game.module.mybatis.entity.UserCard;
import com.zanxiang.game.module.mybatis.mapper.UserCardMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-09-26
 * @description : 用户实名认证
 */
@Slf4j
@Service
public class UserCardServiceImpl extends ServiceImpl<UserCardMapper, UserCard> implements IUserCardService {

    @Autowired
    private IUserService userService;

    @Autowired
    private IAgentService agentService;

    @Override
    public IPage<UserCardVO> getUserCardList(UserNameAuthListParam param) {
        //渠道列表
        List<AgentDTO> agentDTOList = agentService.listUserAgent(null, null, null);
        //根据条件, 匹配渠道
        if (CollectionUtils.isEmpty(agentDTOList) && !SecurityUtil.isManager()) {
            return new Page<>();
        }
        //渠道id
        List<Long> agentIdList = agentDTOList.stream().map(AgentDTO::getId).collect(Collectors.toList());
        //管理员可见自然量数据
        if (SecurityUtil.isManager()) {
            agentIdList.add(0L);
        }
        //玩家条件处理
        Map<Long, UserDTO> userMap = userService.userCondition(param.getUserId(), agentIdList, param.getUserName(),
                param.getNickname(), null);
        //根据条件, 匹配不到玩家
        if (userMap != null && userMap.isEmpty()) {
            return new Page<>();
        }
        Set<Long> userIdSet = userMap == null ? null : userMap.keySet();
        return page(param.toPage(), new QueryWrapper<UserCard>().lambda()
                .in(CollectionUtils.isNotEmpty(userIdSet), UserCard::getUserId, userIdSet)
                .eq(Strings.isNotBlank(param.getUserName()), UserCard::getUsername, param.getUserName())
                .eq(Strings.isNotBlank(param.getNickname()), UserCard::getNickname, param.getNickname())
                .ge(param.getBeginDate() != null, UserCard::getRegTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, UserCard::getRegTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .eq(Strings.isNotBlank(param.getRealName()), UserCard::getCardName, param.getRealName())
                .orderByDesc(UserCard::getCreateTime))
                .convert(this::toVo);
    }

    private UserCardVO toVo(UserCard userCard) {
        if (Objects.isNull(userCard)) {
            return null;
        }
        UserCardVO userCardVO = BeanUtil.copy(userCard, UserCardVO.class);
        userCardVO.setCardId(userCard.getShowCardId());
        userCardVO.setCardName(userCard.getShowCardName());
        return userCardVO;
    }

    @Override
    public UserCardVO getByUserId(Long userId) {
        UserCard userCard = this.getOne(new LambdaQueryWrapper<UserCard>()
                .eq(UserCard::getUserId, userId));
        return this.toVo(userCard);
    }

}
