package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.base.pojo.enums.GameCategoryEnum;
import com.zanxiang.game.module.manage.pojo.dto.AgentDTO;
import com.zanxiang.game.module.manage.pojo.dto.CpDTO;
import com.zanxiang.game.module.manage.pojo.dto.GameDTO;
import com.zanxiang.game.module.manage.pojo.dto.UserDTO;
import com.zanxiang.game.module.manage.pojo.params.UserListParam;
import com.zanxiang.game.module.manage.pojo.params.UserUpdateParam;
import com.zanxiang.game.module.manage.pojo.vo.UserCardVO;
import com.zanxiang.game.module.manage.pojo.vo.UserListVO;
import com.zanxiang.game.module.manage.pojo.vo.UserVO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.Agent;
import com.zanxiang.game.module.mybatis.entity.User;
import com.zanxiang.game.module.mybatis.mapper.UserMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.encryption.Md5Util;
import com.zanxiang.module.util.exception.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户玩家
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private IGameService gameService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IUserCardService userCardService;

    @Autowired
    private IAgentService agentService;

    @Override
    public IPage<UserListVO> list(UserListParam param) {
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        //渠道列表
        List<AgentDTO> agentDTOList = agentService.listUserAgent(param.getAccountId(), param.getPitcherId(), param.getChannelId());
        //根据条件, 匹配渠道
        if (CollectionUtils.isEmpty(agentDTOList) && !SecurityUtil.isManager()) {
            return new Page<>();
        }
        //渠道id列表
        List<Long> agentIdList = agentDTOList.stream().map(AgentDTO::getId).collect(Collectors.toList());
        //管理员可见自然量数据
        if (SecurityUtil.isManager()) {
            agentIdList.add(0L);
        }
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        Map<Long, AgentDTO> agentMap = agentDTOList.stream().collect(Collectors.toMap(AgentDTO::getId, Function.identity()));
        //执行查询
        return page(param.toPage(), new QueryWrapper<User>().lambda()
                .eq(param.getUserId() != null, User::getId, param.getUserId())
                .like(Strings.isNotBlank(param.getUserName()), User::getUsername, param.getUserName())
                .like(Strings.isNotBlank(param.getNickname()), User::getNickname, param.getNickname())
                .eq(Strings.isNotBlank(param.getMobile()), User::getMobile, param.getMobile())
                .eq(Strings.isNotBlank(param.getRegIp()), User::getIp, param.getRegIp())
                .eq(Objects.equals(param.getIsAuth(), Boolean.FALSE), User::getAuthentication, 0)
                .ne(Objects.equals(param.getIsAuth(), Boolean.TRUE), User::getAuthentication, 0)
                .isNotNull(Objects.equals(param.getIsBindMobile(), Boolean.TRUE), User::getMobile)
                .isNull(Objects.equals(param.getIsBindMobile(), Boolean.FALSE), User::getMobile)
                .in(CollectionUtils.isNotEmpty(gameMap), User::getGameId, CollectionUtils.isEmpty(gameMap) ? null : gameMap.keySet())
                .ge(param.getBeginDate() != null, User::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, User::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, User::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, User::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), User::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), User::getRechargeCount, 0)
                .eq(param.getStatus() != null, User::getStatus, param.getStatus())
                .in(CollectionUtils.isNotEmpty(agentIdList), User::getAgentId, agentIdList)
                .orderByDesc(User::getCreateTime)
        ).convert(u -> this.toVo(u, gameMap, cpMap, agentMap));
    }

    private UserListVO toVo(User user, Map<Long, GameDTO> gameMap, Map<Long, CpDTO> cpMap, Map<Long, AgentDTO> agentMap) {
        UserListVO userListVO = BeanUtil.copy(user, UserListVO.class);
        if (Objects.isNull(userListVO)) {
            return null;
        }
        //手机号隐藏
        userListVO.setMobile(user.getShowPhoneNum());
        userListVO.setRegIp(user.getIp());
        //渠道信息
        AgentDTO agentDTO = agentMap.get(userListVO.getAgentId());
        if (agentDTO != null) {
            userListVO.setAgentName(agentDTO.getAgentName());
            userListVO.setAccountId(agentDTO.getId());
            userListVO.setPitcherId(agentDTO.getCreateBy());
            userListVO.setPitcherName(agentDTO.getCreateByName());
            userListVO.setAccountType(agentDTO.getAccountType());
        }
        //游戏信息
        GameDTO gameDTO = CollectionUtils.isNotEmpty(gameMap) ? gameMap.get(userListVO.getGameId()) : gameService.getById(userListVO.getGameId());
        if (gameDTO != null) {
            userListVO.setGameName(gameDTO.getName());
            userListVO.setGameCategoryId(gameDTO.getCategory());
            userListVO.setGameCategoryName(GameCategoryEnum.getNameByCategory(gameDTO.getCategory()));
            userListVO.setCpId(gameDTO.getCpId());
            userListVO.setCpName(cpMap.get(gameDTO.getCpId()) == null ? null : cpMap.get(gameDTO.getCpId()).getCpName());
        }
        return userListVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean userUpdate(UserUpdateParam param) {
        //用户信息更新
        if (Strings.isNotBlank(param.getMobile()) || Strings.isNotBlank(param.getAliPay()) || Strings.isNotBlank(param.getPassword())) {
            super.update(new LambdaUpdateWrapper<User>()
                    .eq(User::getId, param.getUserId())
                    .eq(User::getUsername, param.getUsername())
                    .set(Strings.isNotBlank(param.getMobile()) && !param.getMobile().contains("*"), User::getMobile, param.getMobile())
                    .set(Strings.isNotBlank(param.getAliPay()), User::getAliPay, param.getAliPay())
                    .set(Strings.isNotBlank(param.getPassword()), User::getPassword, this.cmfPassword(param.getPassword()))
                    .set(User::getUpdateTime, LocalDateTime.now()));
        }
        return Boolean.TRUE;
    }

    private String cmfPassword(String password) {
        String passwordSalt = password + "ZX_PASSWORD_SALT";
        return "###" + Md5Util.encrypt32(Md5Util.encrypt32(passwordSalt));
    }

    @Override
    public UserVO getUserInfo(Long userId) {
        //查询用户信息
        User user = super.getById(userId);
        UserVO userVO = BeanUtil.copy(user, UserVO.class);
        if (userVO == null) {
            throw new BaseException("用户信息不存在!");
        }
        //查询用户拓展信息
        UserCardVO userCardVO = userCardService.getByUserId(userId);
        //拓展信息补充
        userVO.setRealName(userCardVO == null ? null : userCardVO.getCardName());
        userVO.setBirthday(userCardVO == null ? null : userCardVO.getBirthday());
        userVO.setSex(userCardVO == null ? null : userCardVO.getSex());
        userVO.setIdCard(userCardVO == null ? null : userCardVO.getCardId());
        userVO.setRegMobile(user.getShowPhoneNum());
        userVO.setMobile(user.getShowPhoneNum());
        userVO.setRegEmail(user.getEmail());
        userVO.setRegIp(user.getIp());
        userVO.setDeviceId(user.getMac());
        //注册游戏
        GameDTO gameDTO = gameService.getById(user.getGameId());
        if (gameDTO != null) {
            userVO.setRegGame(gameDTO.getName());
        }
        //渠道
        Agent agent = agentService.getById(user.getAgentId());
        if (agent != null) {
            userVO.setChannelName(agent.getAgentName());
            userVO.setRegFrom(agent.getAccountType());
        }
        return userVO;
    }

    @Override
    public Map<Long, UserDTO> userCondition(Long userId, List<Long> agentIdList, String userName, String nickname, String regIp) {
        //用户信息条件处理
        if (userId == null && CollectionUtils.isEmpty(agentIdList) && Strings.isBlank(userName)
                && Strings.isBlank(nickname) && Strings.isBlank(regIp)) {
            return null;
        }
        //查询
        List<User> userList = super.list(new LambdaQueryWrapper<User>()
                .eq(userId != null, User::getId, userId)
                .in(CollectionUtils.isNotEmpty(agentIdList), User::getAgentId, agentIdList)
                .eq(Strings.isNotBlank(regIp), User::getIp, regIp)
                .like(Strings.isNotBlank(userName), User::getUsername, userName)
                .like(Strings.isNotBlank(nickname), User::getNickname, nickname)
                .select(User::getId, User::getUsername, User::getNickname, User::getIp, User::getCreateTime, User::getAgentId));
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        return userList.stream()
                .map(user -> BeanUtil.copy(user, UserDTO.class))
                .collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    @Override
    public Map<Long, UserDTO> getByUserIds(List<Long> userIdList) {
        if (CollectionUtils.isEmpty(userIdList)) {
            return Collections.emptyMap();
        }
        //查询
        List<User> userList = super.listByIds(userIdList);
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        return userList.stream().map(user -> BeanUtil.copy(user, UserDTO.class))
                .collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    @Override
    public UserDTO getById(Long userId) {
        return BeanUtil.copy(super.getById(userId), UserDTO.class);
    }

}
